/*
 * Copyright (C) 2003 Patrick Reynolds <reynolds@cs.duke.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 *      Free Software Foundation, Inc.
 *      59 Temple Place
 *      Suite 330
 *      Boston, MA 02111-1307
 *      USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libppm.h"

#define CHK(a, b) if (!(a)) { fprintf(stderr, "%s failed\n", (b)); return -1; }

int abs(int n) {
	return (n >= 0) ? n : -n;
}

int main(int argc, char *argv[]) {
  struct ppm_image imga, imgb;
	int x, pixels;
	int total=0, totalsq=0, thresh[256], threshtotal=0;
	FILE *fp;

  if (argc != 3) {
    fprintf(stderr, "Usage:\n  ppmscale a.ppm b.ppm\n\n");
    exit(1);
  }
  CHK(ppminit(&imga) >= 0, "ppminit")
  CHK(ppminit(&imgb) >= 0, "ppminit")
	CHK((fp = fopen(argv[1], "r")) != NULL, argv[1])
  CHK(ppmread(fp, &imga) >= 0, "ppmread")
	fclose(fp);
	CHK((fp = fopen(argv[2], "r")) != NULL, argv[2])
  CHK(ppmread(fp, &imgb) >= 0, "ppmread")
	fclose(fp);

	if (imga.width != imgb.width || imga.height != imgb.height) {
		fprintf(stderr, "Image sizes unequal: %dx%d vs %dx%d\n",
			imga.width, imgb.width, imga.height, imgb.height);
		return -1;
	}

	pixels = imga.width * imga.height;
	for (x=0; x<256; x++) thresh[x] = 0;
	for (x=pixels*3-1; x>=0; x--) {
		int t = abs(imga.data[x] - imgb.data[x]);
		total += t;
		totalsq += t*t;
		imga.data[x] = t;
		thresh[t]++;
	}

	fprintf(stderr, "Per-pixel difference: %.5f\n", (double)total/pixels);
	fprintf(stderr, "Per-pixel sqdifference: %.5f\n", (double)totalsq/pixels);
	fprintf(stderr, "N   Pixels changed >= N\n");
	for (x=255; x>=0; x--)
		if (thresh[x] > 0)
			fprintf(stderr, "%3d %d\n", x, threshtotal+=thresh[x]);
  CHK(ppmwrite(stdout, &imga) >= 0, "ppmwrite")
  ppmdestroy(&imga);
  ppmdestroy(&imgb);

  return 0;
}
